# Flask_before_first_request和before_request详解：
import os

from flask import Flask, session, g, render_template, abort

app = Flask(__name__)
app.config['SECRET_KEY'] = os.urandom(24)


@app.route('/')
def hello_world():
    session['uname'] = 'momo'
    # print(g.user)
    # return 'Hello World!'
    return render_template('index.html')


@app.route('/list/')
def myli():
    print('mylist')
    if hasattr(g, 'user'):
        print('条件取出', g.user)
    # return 'hello world'
    return render_template('list.html')


# 1.before_first_request：处理项目的第一次请求之前执行。
# @app.before_first_request
# def first_request():
#     print('hi')

# 2.before_request：在每次请求之前执行。通常可以用这个装饰器来给视图函数增加一些变量。
# 请求已经到达了Flask，但是还没有进入到具体的视图函数之前调用。一般这个就是在视图函数之前，
# 我们可以把一些后面需要用到的数据先处理好，方便视图函数使用。
@app.before_request
def before_request():
    print('在视图函数执行之前执行的钩子函数')
    # 场景：若用户已经登录了，验证时把用户名放入session中,之后取出来，放入钩子函数，以后访问的视图函数中可直接取出来使用
    uname = session.get('uname')
    print(uname)
    if uname:
        g.user = uname


# context_processor：上下文处理器。使用这个钩子函数，必须返回一个字典。
# 这个字典中的值在所有模版中都可以使用。
# 这个钩子函数的函数是，如果一些在很多模版中都要用到的变量，那么就可以使用这个钩子函数来返回，而不用在每个视图函数中的`render_template`中去写，这样可以让代码更加简洁和好维护。
@app.context_processor
def context_processor():
    if hasattr(g, 'user'):
        return {'current_user': g.user}
    else:
        return {}


# errorhandler：errorhandler接收状态码，可以自定义返回这种状态码的响应的处理方法。在发生一些异常的时候，比如404错误，比如500错误,那么如果想要优雅的处理这些错误，就可以使用`errorhandler`来出来。
# 需要注意几点：
#     * 在errorhandler装饰的钩子函数下，记得要返回相应的状态码。
#     * 在errorhandler装饰的钩子函数中，必须要写一个参数，来接收错误的信息，如果没有参数，就会直接报错。
#     * 使用`flask.abort`可以手动的抛出相应的错误，比如开发者在发现参数不正确的时候可以自己手动的抛出一个400错误。
@app.errorhandler(500)
def server_error(error):
    print(error)
    return render_template('500.html'), 500


# Flask中的abort函数可以手动的抛出相应的错误（如400）：
@app.errorhandler(400)
def args_error(error):
    return render_template('400.html'), 400


# 该功能需要登录才能设置
@app.route('/setting/')
def setting():
    uname = session.get('uname')
    if uname:
        return '欢迎来到设置页面'
    else:
        abort(400)
    return render_template('list.html')


# 常见的404错误处理：
@app.errorhandler(404)
def page_not_find(error):
    return render_template('404.html'), 404


if __name__ == '__main__':
    app.run()
